@@ -1,21 +1,9 @@
 /*
  *  editdlgtypes.cpp  -  dialogs to create or edit alarm or alarm template types
  *  Program:  kalarm
- *  Copyright © 2001-2020 David Jarvie <djarvie@kde.org>
+ *  SPDX-FileCopyrightText: 2001-2020 David Jarvie <djarvie@kde.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "editdlgtypes.h"
@@ -28,9 +16,8 @@
 #include "kamail.h"
 #include "latecancel.h"
 #include "mainwindow.h"
-#include "messagewin.h"
+#include "messagewindow.h"
 #include "pickfileradio.h"
-#include "preferences.h"
 #include "reminder.h"
 #include "soundpicker.h"
 #include "sounddlg.h"
@@ -73,6 +60,7 @@ using namespace KCalendarCore;
 using namespace KAlarmCal;
 
 enum { tTEXT, tFILE, tCOMMAND };  // order of mTypeCombo items
+enum { dWINDOW, dNOTIFY };        // order of mDisplayMethodCombo items
 
 
 /*=============================================================================
@@ -98,9 +86,11 @@ class PickLogFileRadio : public PickFileRadio
 = Dialog to edit display alarms.
 =============================================================================*/
 
+QString EditDisplayAlarmDlg::i18n_lbl_DisplayMethod() { return i18nc("@label:listbox How to display alarms", "Display method:"); }
+QString EditDisplayAlarmDlg::i18n_combo_Window()      { return i18nc("@item:inlistbox", "Window"); }
+QString EditDisplayAlarmDlg::i18n_combo_Notify()      { return i18nc("@item:inlistbox", "Notification"); }
 QString EditDisplayAlarmDlg::i18n_chk_ConfirmAck()    { return i18nc("@option:check", "Confirm acknowledgment"); }
 
-
 /******************************************************************************
 * Constructor.
 * Parameters:
@@ -140,16 +130,16 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
     // Display type combo box
     QWidget* box = new QWidget(parent);    // to group widgets for QWhatsThis text
 
-    QHBoxLayout* boxHLayout = new QHBoxLayout(box);
+    auto* boxHLayout = new QHBoxLayout(box);
     boxHLayout->setContentsMargins(0, 0, 0, 0);
     QLabel* label = new QLabel(i18nc("@label:listbox", "Display type:"), box);
     boxHLayout->addWidget(label);
     label->setFixedSize(label->sizeHint());
     mTypeCombo = new ComboBox(box);
     boxHLayout->addWidget(mTypeCombo);
-    QString textItem    = i18nc("@item:inlistbox", "Text message");
-    QString fileItem    = i18nc("@item:inlistbox", "File contents");
-    QString commandItem = i18nc("@item:inlistbox", "Command output");
+    const QString textItem    = i18nc("@item:inlistbox", "Text message");
+    const QString fileItem    = i18nc("@item:inlistbox", "File contents");
+    const QString commandItem = i18nc("@item:inlistbox", "Command output");
     mTypeCombo->addItem(textItem);     // index = tTEXT
     mTypeCombo->addItem(fileItem);     // index = tFILE
     mTypeCombo->addItem(commandItem);  // index = tCOMMAND
@@ -158,7 +148,7 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
     if (!ShellProcess::authorised())
     {
         // User not authorised to issue shell commands - disable Command Output option
-        QStandardItemModel* model = qobject_cast<QStandardItemModel*>(mTypeCombo->model());
+        auto* model = qobject_cast<QStandardItemModel*>(mTypeCombo->model());
         if (model)
         {
             QModelIndex index = model->index(2, mTypeCombo->modelColumn(), mTypeCombo->rootModelIndex());
@@ -171,12 +161,15 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
     connect(mTypeCombo, static_cast<void (ComboBox::*)(int)>(&ComboBox::currentIndexChanged), this, &EditDisplayAlarmDlg::contentsChanged);
     label->setBuddy(mTypeCombo);
     box->setWhatsThis(xi18nc("@info:whatsthis", "<para>Select what the alarm should display:"
-          "<list><item><interface>%1</interface>: the alarm will display the text message you type in.</item>"
-          "<item><interface>%2</interface>: the alarm will display the contents of a text or image file.</item>"
-          "<item><interface>%3</interface>: the alarm will display the output from a command.</item></list></para>",
-          textItem, fileItem, commandItem));
-    boxHLayout->setStretchFactor(new QWidget(box), 1);    // left adjust the control
-    frameLayout->addWidget(box);
+                             "<list><item><interface>%1</interface>: the alarm will display the text message you type in.</item>"
+                             "<item><interface>%2</interface>: the alarm will display the contents of a text or image file.</item>"
+                             "<item><interface>%3</interface>: the alarm will display the output from a command.</item></list></para>",
+                             textItem, fileItem, commandItem));
+    auto* hlayout = new QHBoxLayout();
+    hlayout->setContentsMargins(0, 0, 0, 0);
+    frameLayout->addLayout(hlayout);
+    hlayout->addWidget(box);
+    hlayout->addStretch();    // left adjust the control
 
     // Text message edit box
     mTextMessageEdit = new TextEdit(parent);
@@ -189,7 +182,7 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
     // File name edit box
     mFileBox = new QWidget(parent);
     frameLayout->addWidget(mFileBox);
-    QHBoxLayout* fileBoxHLayout = new QHBoxLayout(mFileBox);
+    auto* fileBoxHLayout = new QHBoxLayout(mFileBox);
     fileBoxHLayout->setContentsMargins(0, 0, 0, 0);
     fileBoxHLayout->setSpacing(0);
     mFileMessageEdit = new LineEdit(LineEdit::Url, mFileBox);
@@ -215,7 +208,7 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
     frameLayout->addWidget(mCmdEdit);
 
     // Sound checkbox and file selector
-    QHBoxLayout* hlayout = new QHBoxLayout();
+    hlayout = new QHBoxLayout();
     hlayout->setContentsMargins(0, 0, 0, 0);
     frameLayout->addLayout(hlayout);
     mSoundPicker = new SoundPicker(parent);
@@ -232,13 +225,38 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
     connect(mFontColourButton, &FontColourButton::selected, this, &EditDisplayAlarmDlg::setColours);
     connect(mFontColourButton, &FontColourButton::selected, this, &EditDisplayAlarmDlg::contentsChanged);
 
+    // Display method selector
+    hlayout = new QHBoxLayout();
+    hlayout->setContentsMargins(0, 0, 0, 0);
+    frameLayout->addLayout(hlayout);
+    mDisplayMethodBox = new QWidget(parent);    // to group widgets for QWhatsThis text
+    boxHLayout = new QHBoxLayout(mDisplayMethodBox);
+    boxHLayout->setContentsMargins(0, 0, 0, 0);
+    label = new QLabel(i18n_lbl_DisplayMethod(), mDisplayMethodBox);
+    boxHLayout->addWidget(label);
+    label->setFixedSize(label->sizeHint());
+    mDisplayMethodCombo = new ComboBox(mDisplayMethodBox);
+    boxHLayout->addWidget(mDisplayMethodCombo);
+    const QString windowItem = i18n_combo_Window();
+    const QString notifyItem = i18n_combo_Notify();
+    mDisplayMethodCombo->addItem(windowItem);     // index = dWINDOW
+    mDisplayMethodCombo->addItem(notifyItem);     // index = dNOTIFY
+    mDisplayMethodCombo->setFixedSize(mDisplayMethodCombo->sizeHint());
+    connect(mDisplayMethodCombo, static_cast<void (ComboBox::*)(int)>(&ComboBox::currentIndexChanged), this, &EditDisplayAlarmDlg::slotDisplayMethodChanged);
+    connect(mDisplayMethodCombo, static_cast<void (ComboBox::*)(int)>(&ComboBox::currentIndexChanged), this, &EditDisplayAlarmDlg::contentsChanged);
+    label->setBuddy(mDisplayMethodCombo);
+    mDisplayMethodBox->setWhatsThis(i18nc("@info:whatsthis", "Select whether to display the alarm in a window or by the notification system."));
+    hlayout->addWidget(mDisplayMethodBox);
+    hlayout->addSpacing(2 * style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing));
+    hlayout->addStretch();
+
     if (ShellProcess::authorised())    // don't display if shell commands not allowed (e.g. kiosk mode)
     {
         // Special actions button
         mSpecialActionsButton = new SpecialActionsButton(false, parent);
         mSpecialActionsButton->setFixedSize(mSpecialActionsButton->sizeHint());
         connect(mSpecialActionsButton, &SpecialActionsButton::selected, this, &EditDisplayAlarmDlg::contentsChanged);
-        frameLayout->addWidget(mSpecialActionsButton, 0, Qt::AlignRight);
+        hlayout->addWidget(mSpecialActionsButton);
     }
 
     // Top-adjust the controls
@@ -276,6 +294,7 @@ CheckBox* EditDisplayAlarmDlg::createConfirmAckCheckbox(QWidget* parent)
 */
 void EditDisplayAlarmDlg::type_initValues(const KAEvent* event)
 {
+    mTryButton->setToolTip(i18nc("@info:tooltip", "Display the alarm now"));
     mAkonadiItemId = -1;
     lateCancel()->showAutoClose(true);
     if (event)
@@ -291,8 +310,9 @@ void EditDisplayAlarmDlg::type_initValues(const KAEvent* event)
         mFontColourButton->setBgColour(event->bgColour());
         mFontColourButton->setFgColour(event->fgColour());
         setColours(event->fgColour(), event->bgColour());
+        mDisplayMethodCombo->setCurrentIndex(event->notify() ? dNOTIFY : dWINDOW);
         mConfirmAck->setChecked(event->confirmAck());
-        bool recurs = event->recurs();
+        const bool recurs = event->recurs();
         int reminderMins = event->reminderMinutes();
         if (reminderMins > 0  &&  !event->reminderActive())
             reminderMins = 0;   // don't show advance reminder which has already passed
@@ -336,6 +356,7 @@ void EditDisplayAlarmDlg::type_initValues(const KAEvent* event)
         mFontColourButton->setBgColour(Preferences::defaultBgColour());
         mFontColourButton->setFgColour(Preferences::defaultFgColour());
         setColours(Preferences::defaultFgColour(), Preferences::defaultBgColour());
+        mDisplayMethodCombo->setCurrentIndex(Preferences::defaultDisplayMethod() == Preferences::Display_Window ? dWINDOW : dNOTIFY);
         mConfirmAck->setChecked(Preferences::defaultConfirmAck());
         reminder()->setMinutes(0, false);
         reminder()->enableOnceOnly(isTimedRecurrence());   // must be called after mRecurrenceEdit is set up
@@ -353,6 +374,7 @@ void EditDisplayAlarmDlg::type_initValues(const KAEvent* event)
         mSoundPicker->set(Preferences::defaultSoundType(), Preferences::defaultSoundFile(),
                           Preferences::defaultSoundVolume(), -1, 0, (Preferences::defaultSoundRepeat() ? 0 : -1));
     }
+    slotDisplayMethodChanged(mDisplayMethodCombo->currentIndex());
 }
 
 /******************************************************************************
@@ -364,9 +386,15 @@ void EditDisplayAlarmDlg::type_showOptions(bool more)
     if (mSpecialActionsButton)
     {
         if (more)
+        {
+            mDisplayMethodBox->show();
             mSpecialActionsButton->show();
+        }
         else
+        {
+            mDisplayMethodBox->hide();
             mSpecialActionsButton->hide();
+        }
     }
 }
 
@@ -376,18 +404,29 @@ void EditDisplayAlarmDlg::type_showOptions(bool more)
 */
 void EditDisplayAlarmDlg::setColours(const QColor& fgColour, const QColor& bgColour)
 {
-    QPalette pal = mTextMessageEdit->palette();
-    pal.setColor(mTextMessageEdit->backgroundRole(), bgColour);
-    pal.setColor(QPalette::Text, fgColour);
-    mTextMessageEdit->setPalette(pal);
-    pal = mTextMessageEdit->viewport()->palette();
-    pal.setColor(mTextMessageEdit->viewport()->backgroundRole(), bgColour);
-    pal.setColor(QPalette::Text, fgColour);
-    mTextMessageEdit->viewport()->setPalette(pal);
+    QColor fg(fgColour);
+    if (mDisplayMethodCombo->currentIndex() == dNOTIFY)
+    {
+        const QPalette pal = mFileMessageEdit->palette();
+        mTextMessageEdit->setPalette(pal);
+        mTextMessageEdit->viewport()->setPalette(pal);
+        fg = pal.color(QPalette::Text);
+    }
+    else
+    {
+        QPalette pal = mTextMessageEdit->palette();
+        pal.setColor(mTextMessageEdit->backgroundRole(), bgColour);
+        pal.setColor(QPalette::Text, fgColour);
+        mTextMessageEdit->setPalette(pal);
+        pal = mTextMessageEdit->viewport()->palette();
+        pal.setColor(mTextMessageEdit->viewport()->backgroundRole(), bgColour);
+        pal.setColor(QPalette::Text, fgColour);
+        mTextMessageEdit->viewport()->setPalette(pal);
+    }
     // Change the color of existing text
-    QTextCursor cursor = mTextMessageEdit->textCursor();
+    const QTextCursor cursor = mTextMessageEdit->textCursor();
     mTextMessageEdit->selectAll();
-    mTextMessageEdit->setTextColor(fgColour);
+    mTextMessageEdit->setTextColor(fg);
     mTextMessageEdit->setTextCursor(cursor);
 }
 
@@ -396,7 +435,7 @@ void EditDisplayAlarmDlg::setColours(const QColor& fgColour, const QColor& bgCol
 */
 void EditDisplayAlarmDlg::setAction(KAEvent::SubAction action, const AlarmText& alarmText)
 {
-    QString text = alarmText.displayText();
+    const QString text = alarmText.displayText();
     switch (action)
     {
         case KAEvent::MESSAGE:
@@ -431,6 +470,10 @@ void EditDisplayAlarmDlg::setFgColour(const QColor& colour)
     mFontColourButton->setFgColour(colour);
     setColours(colour, mFontColourButton->bgColour());
 }
+void EditDisplayAlarmDlg::setNotify(bool notify)
+{
+    mDisplayMethodCombo->setCurrentIndex(notify ? dNOTIFY : dWINDOW);
+}
 void EditDisplayAlarmDlg::setConfirmAck(bool confirm)
 {
     mConfirmAck->setChecked(confirm);
@@ -461,6 +504,7 @@ void EditDisplayAlarmDlg::setReadOnly(bool readOnly)
     mCmdEdit->setReadOnly(readOnly);
     mFontColourButton->setReadOnly(readOnly);
     mSoundPicker->setReadOnly(readOnly);
+    mDisplayMethodCombo->setReadOnly(readOnly);
     mConfirmAck->setReadOnly(readOnly);
     reminder()->setReadOnly(readOnly);
     if (mSpecialActionsButton)
@@ -478,19 +522,20 @@ void EditDisplayAlarmDlg::setReadOnly(bool readOnly)
 void EditDisplayAlarmDlg::saveState(const KAEvent* event)
 {
     EditAlarmDlg::saveState(event);
-    mSavedType        = mTypeCombo->currentIndex();
-    mSavedCmdScript   = mCmdEdit->isScript();
-    mSavedSoundType   = mSoundPicker->sound();
-    mSavedSoundFile   = mSoundPicker->file();
-    mSavedSoundVolume = mSoundPicker->volume(mSavedSoundFadeVolume, mSavedSoundFadeSeconds);
-    mSavedRepeatPause = mSoundPicker->repeatPause();
-    mSavedConfirmAck  = mConfirmAck->isChecked();
-    mSavedFont        = mFontColourButton->font();
-    mSavedFgColour    = mFontColourButton->fgColour();
-    mSavedBgColour    = mFontColourButton->bgColour();
-    mSavedReminder    = reminder()->minutes();
-    mSavedOnceOnly    = reminder()->isOnceOnly();
-    mSavedAutoClose   = lateCancel()->isAutoClose();
+    mSavedType          = mTypeCombo->currentIndex();
+    mSavedCmdScript     = mCmdEdit->isScript();
+    mSavedSoundType     = mSoundPicker->sound();
+    mSavedSoundFile     = mSoundPicker->file();
+    mSavedSoundVolume   = mSoundPicker->volume(mSavedSoundFadeVolume, mSavedSoundFadeSeconds);
+    mSavedRepeatPause   = mSoundPicker->repeatPause();
+    mSavedDisplayMethod = mDisplayMethodCombo->currentIndex();
+    mSavedConfirmAck    = mConfirmAck->isChecked();
+    mSavedFont          = mFontColourButton->font();
+    mSavedFgColour      = mFontColourButton->fgColour();
+    mSavedBgColour      = mFontColourButton->bgColour();
+    mSavedReminder      = reminder()->minutes();
+    mSavedOnceOnly      = reminder()->isOnceOnly();
+    mSavedAutoClose     = lateCancel()->isAutoClose();
     if (mSpecialActionsButton)
     {
         mSavedPreAction        = mSpecialActionsButton->preAction();
@@ -507,17 +552,22 @@ void EditDisplayAlarmDlg::saveState(const KAEvent* event)
 */
 bool EditDisplayAlarmDlg::type_stateChanged() const
 {
-    if (mSavedType       != mTypeCombo->currentIndex()
-    ||  mSavedCmdScript  != mCmdEdit->isScript()
-    ||  mSavedSoundType  != mSoundPicker->sound()
-    ||  mSavedConfirmAck != mConfirmAck->isChecked()
-    ||  mSavedFont       != mFontColourButton->font()
-    ||  mSavedFgColour   != mFontColourButton->fgColour()
-    ||  mSavedBgColour   != mFontColourButton->bgColour()
-    ||  mSavedReminder   != reminder()->minutes()
-    ||  mSavedOnceOnly   != reminder()->isOnceOnly()
-    ||  mSavedAutoClose  != lateCancel()->isAutoClose())
+    if (mSavedType          != mTypeCombo->currentIndex()
+    ||  mSavedCmdScript     != mCmdEdit->isScript()
+    ||  mSavedSoundType     != mSoundPicker->sound()
+    ||  mSavedDisplayMethod != mDisplayMethodCombo->currentIndex()
+    ||  mSavedReminder      != reminder()->minutes()
+    ||  mSavedOnceOnly      != reminder()->isOnceOnly())
         return true;
+    if (mDisplayMethodCombo->currentIndex() == dWINDOW)
+    {
+        if (mSavedConfirmAck != mConfirmAck->isChecked()
+        ||  mSavedFont       != mFontColourButton->font()
+        ||  mSavedFgColour   != mFontColourButton->fgColour()
+        ||  mSavedBgColour   != mFontColourButton->bgColour()
+        ||  mSavedAutoClose  != lateCancel()->isAutoClose())
+            return true;
+    }
     if (mSpecialActionsButton)
     {
         if (mSavedPreAction        != mSpecialActionsButton->preAction()
@@ -547,7 +597,7 @@ bool EditDisplayAlarmDlg::type_stateChanged() const
 * Extract the data in the dialog specific to the alarm type and set up a
 * KAEvent from it.
 */
-void EditDisplayAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, const QString& text, int lateCancel, bool trial)
+void EditDisplayAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, const QString& name, const QString& text, int lateCancel, bool trial)
 {
     KAEvent::SubAction type;
     switch (mTypeCombo->currentIndex())
@@ -557,8 +607,20 @@ void EditDisplayAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, co
         default:
         case tTEXT:     type = KAEvent::MESSAGE; break;
     }
-    event = KAEvent(dt, text, mFontColourButton->bgColour(), mFontColourButton->fgColour(),
-                    mFontColourButton->font(), type, lateCancel, getAlarmFlags());
+    QColor fgColour, bgColour;
+    QFont font;
+    if (mDisplayMethodCombo->currentIndex() == dNOTIFY)
+    {
+        bgColour = Preferences::defaultBgColour();
+        fgColour = Preferences::defaultFgColour();
+    }
+    else
+    {
+        bgColour = mFontColourButton->bgColour();
+        fgColour = mFontColourButton->fgColour();
+        font     = mFontColourButton->font();
+    }
+    event = KAEvent(dt, name, text, bgColour, fgColour, font, type, lateCancel, getAlarmFlags());
     if (type == KAEvent::MESSAGE)
     {
         if (AlarmText::checkIfEmail(text))
@@ -566,8 +628,8 @@ void EditDisplayAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, co
     }
     float fadeVolume;
     int   fadeSecs;
-    float volume = mSoundPicker->volume(fadeVolume, fadeSecs);
-    int   repeatPause = mSoundPicker->repeatPause();
+    const float volume = mSoundPicker->volume(fadeVolume, fadeSecs);
+    const int   repeatPause = mSoundPicker->repeatPause();
     event.setAudioFile(mSoundPicker->file().toDisplayString(), volume, fadeVolume, fadeSecs, repeatPause);
     if (!trial  &&  reminder()->isEnabled())
         event.setReminder(reminder()->minutes(), reminder()->isOnceOnly());
@@ -581,22 +643,30 @@ void EditDisplayAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, co
 */
 KAEvent::Flags EditDisplayAlarmDlg::getAlarmFlags() const
 {
-    bool cmd = (mTypeCombo->currentIndex() == tCOMMAND);
+    const bool cmd = (mTypeCombo->currentIndex() == tCOMMAND);
     KAEvent::Flags flags = EditAlarmDlg::getAlarmFlags();
     if (mSoundPicker->sound() == Preferences::Sound_Beep)  flags |= KAEvent::BEEP;
     if (mSoundPicker->sound() == Preferences::Sound_Speak) flags |= KAEvent::SPEAK;
     if (mSoundPicker->repeatPause() >= 0)                  flags |= KAEvent::REPEAT_SOUND;
-    if (mConfirmAck->isChecked())                          flags |= KAEvent::CONFIRM_ACK;
-    if (lateCancel()->isAutoClose())                       flags |= KAEvent::AUTO_CLOSE;
-    if (mFontColourButton->defaultFont())                  flags |= KAEvent::DEFAULT_FONT;
     if (cmd)                                               flags |= KAEvent::DISPLAY_COMMAND;
     if (cmd && mCmdEdit->isScript())                       flags |= KAEvent::SCRIPT;
+    if (mDisplayMethodCombo->currentIndex() == dNOTIFY)
+    {
+                                                           flags |= KAEvent::NOTIFY;
+                                                           flags |= KAEvent::DEFAULT_FONT;
+    }
+    else
+    {
+        if (mFontColourButton->defaultFont())              flags |= KAEvent::DEFAULT_FONT;
+        if (mConfirmAck->isChecked())                      flags |= KAEvent::CONFIRM_ACK;
+        if (lateCancel()->isAutoClose())                   flags |= KAEvent::AUTO_CLOSE;
+    }
     return flags;
 }
 
 /******************************************************************************
-* Called when one of the alarm display type combo box is changed, to display
-* the appropriate set of controls for that action type.
+* Called when the alarm display type combo box is changed, to display the
+* appropriate set of controls for that action type.
 */
 void EditDisplayAlarmDlg::slotAlarmTypeChanged(int index)
 {
@@ -636,6 +706,23 @@ void EditDisplayAlarmDlg::slotAlarmTypeChanged(int index)
         focus->setFocus();
 }
 
+/******************************************************************************
+* Called when the display method combo box is changed, to enable/disable the
+* appropriate set of controls for that display method.
+*/
+void EditDisplayAlarmDlg::slotDisplayMethodChanged(int index)
+{
+    const bool enable = (index == dWINDOW);
+    mConfirmAck->setVisible(enable);
+    mFontColourButton->setVisible(enable);
+    mSoundPicker->showFile(enable);
+    // Because notifications automatically time out after 10 seconds,
+    // auto-close would always occur after a notification closes.
+    lateCancel()->showAutoClose(enable);
+    // Set the text message edit box colours according to the display metho according to the display methodd.
+    setColours(mFontColourButton->fgColour(), mFontColourButton->bgColour());
+}
+
 /******************************************************************************
 * Called when the file browse button is pressed to select a file to display.
 */
@@ -763,6 +850,7 @@ QString EditCommandAlarmDlg::type_caption() const
 void EditCommandAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 {
     mTryButton->setWhatsThis(i18nc("@info:whatsthis", "Execute the specified command now"));
+    mTryButton->setToolTip(i18nc("@info:tooltip", "Execute the specified command now"));
 
     mCmdEdit = new CommandEdit(parent);
     connect(mCmdEdit, &CommandEdit::scriptToggled, this, &EditCommandAlarmDlg::slotCmdScriptToggled);
@@ -778,7 +866,7 @@ void EditCommandAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 
     mCmdOutputBox = new QGroupBox(i18nc("@title:group", "Command Output"), parent);
     frameLayout->addWidget(mCmdOutputBox);
-    QVBoxLayout* vlayout = new QVBoxLayout(mCmdOutputBox);
+    auto* vlayout = new QVBoxLayout(mCmdOutputBox);
     mCmdOutputGroup = new ButtonGroup(mCmdOutputBox);
     connect(mCmdOutputGroup, &ButtonGroup::buttonSet, this, &EditCommandAlarmDlg::contentsChanged);
 
@@ -791,7 +879,7 @@ void EditCommandAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 
     // Log file name edit box
     QWidget* box = new QWidget(mCmdOutputBox);
-    QHBoxLayout* boxHLayout = new QHBoxLayout(box);
+    auto* boxHLayout = new QHBoxLayout(box);
     boxHLayout->setContentsMargins(0, 0, 0, 0);
     boxHLayout->setSpacing(0);
     (new QWidget(box))->setFixedWidth(mCmdExecInTerm->style()->pixelMetric(QStyle::PM_ExclusiveIndicatorWidth));   // indent the edit box
@@ -803,7 +891,7 @@ void EditCommandAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 
     // Log file browse button.
     // The file browser dialog is activated by the PickLogFileRadio class.
-    QPushButton* browseButton = new QPushButton(box);
+    auto* browseButton = new QPushButton(box);
     boxHLayout->addWidget(browseButton);
     browseButton->setIcon(QIcon::fromTheme(QStringLiteral("document-open")));
     int size = browseButton->sizeHint().height();
@@ -829,7 +917,7 @@ void EditCommandAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 
     // Top-adjust the controls
     mCmdPadding = new QWidget(parent);
-    QHBoxLayout* hlayout = new QHBoxLayout(mCmdPadding);
+    auto* hlayout = new QHBoxLayout(mCmdPadding);
     hlayout->setContentsMargins(0, 0, 0, 0);
     hlayout->setSpacing(0);
     frameLayout->addWidget(mCmdPadding);
@@ -936,10 +1024,10 @@ bool EditCommandAlarmDlg::type_stateChanged() const
 * Extract the data in the dialog specific to the alarm type and set up a
 * KAEvent from it.
 */
-void EditCommandAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, const QString& text, int lateCancel, bool trial)
+void EditCommandAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, const QString& name, const QString& text, int lateCancel, bool trial)
 {
     Q_UNUSED(trial);
-    event = KAEvent(dt, text, QColor(), QColor(), QFont(), KAEvent::COMMAND, lateCancel, getAlarmFlags());
+    event = KAEvent(dt, name, text, QColor(), QColor(), QFont(), KAEvent::COMMAND, lateCancel, getAlarmFlags());
     if (mCmdOutputGroup->checkedButton() == mCmdLogToFile)
         event.setLogFile(mCmdLogFileEdit->text());
 }
@@ -965,8 +1053,8 @@ bool EditCommandAlarmDlg::type_validate(bool trial)
     if (mCmdOutputGroup->checkedButton() == mCmdLogToFile)
     {
         // Validate the log file name
-        QString file = mCmdLogFileEdit->text();
-        QFileInfo info(file);
+        const QString file = mCmdLogFileEdit->text();
+        const QFileInfo info(file);
         QDir::setCurrent(QDir::homePath());
         bool err = file.isEmpty()  ||  info.isDir();
         if (!err)
@@ -977,7 +1065,7 @@ bool EditCommandAlarmDlg::type_validate(bool trial)
             }
             else
             {
-                QFileInfo dirinfo(info.absolutePath());    // get absolute directory path
+                const QFileInfo dirinfo(info.absolutePath());    // get absolute directory path
                 err = (!dirinfo.isDir()  ||  !dirinfo.isWritable());
             }
         }
@@ -1010,7 +1098,7 @@ bool EditCommandAlarmDlg::type_validate(bool trial)
 */
 void EditCommandAlarmDlg::type_executedTry(const QString& text, void* result)
 {
-    ShellProcess* proc = (ShellProcess*)result;
+    auto* proc = (ShellProcess*)result;
     if (proc  &&  proc != (void*)-1
     &&  mCmdOutputGroup->checkedButton() != mCmdExecInTerm)
     {
@@ -1089,8 +1177,9 @@ QString EditEmailAlarmDlg::type_caption() const
 void EditEmailAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 {
     mTryButton->setWhatsThis(i18nc("@info:whatsthis", "Send the email to the specified addressees now"));
+    mTryButton->setToolTip(i18nc("@info:tooltip", "Send the email now"));
 
-    QGridLayout* grid = new QGridLayout();
+    auto* grid = new QGridLayout();
     grid->setContentsMargins(0, 0, 0, 0);
     grid->setColumnStretch(1, 1);
     frameLayout->addLayout(grid);
@@ -1214,7 +1303,7 @@ void EditEmailAlarmDlg::type_initValues(const KAEvent* event)
 */
 void EditEmailAlarmDlg::attachmentEnable()
 {
-    bool enable = mEmailAttachList->count();
+    const bool enable = mEmailAttachList->count();
     mEmailAttachList->setEnabled(enable);
     if (mEmailRemoveButton)
         mEmailRemoveButton->setEnabled(enable);
@@ -1326,11 +1415,11 @@ bool EditEmailAlarmDlg::type_stateChanged() const
 * Extract the data in the dialog specific to the alarm type and set up a
 * KAEvent from it.
 */
-void EditEmailAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, const QString& text, int lateCancel, bool trial)
+void EditEmailAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, const QString& name, const QString& text, int lateCancel, bool trial)
 {
     Q_UNUSED(trial);
-    event = KAEvent(dt, text, QColor(), QColor(), QFont(), KAEvent::EMAIL, lateCancel, getAlarmFlags());
-    uint from = mEmailFromList ? mEmailFromList->currentIdentity() : 0;
+    event = KAEvent(dt, name, text, QColor(), QColor(), QFont(), KAEvent::EMAIL, lateCancel, getAlarmFlags());
+    const uint from = mEmailFromList ? mEmailFromList->currentIdentity() : 0;
     event.setEmail(from, mEmailAddresses, mEmailSubjectEdit->text(), mEmailAttachments);
 }
 
@@ -1350,12 +1439,12 @@ KAEvent::Flags EditEmailAlarmDlg::getAlarmFlags() const
 */
 bool EditEmailAlarmDlg::type_validate(bool trial)
 {
-    QString addrs = mEmailToEdit->text();
+    const QString addrs = mEmailToEdit->text();
     if (addrs.isEmpty())
         mEmailAddresses.clear();
     else
     {
-        QString bad = KAMail::convertAddresses(addrs, mEmailAddresses);
+        const QString bad = KAMail::convertAddresses(addrs, mEmailAddresses);
         if (!bad.isEmpty())
         {
             mEmailToEdit->setFocus();
@@ -1436,7 +1525,7 @@ void EditEmailAlarmDlg::openAddressBook()
     Akonadi::EmailAddressSelection::List selections = dlg->selectedAddresses();
     if (selections.isEmpty())
         return;
-    Person person(selections.first().name(), selections.first().email());
+    const Person person(selections.first().name(), selections.first().email());
     QString addrs = mEmailToEdit->text().trimmed();
     if (!addrs.isEmpty())
         addrs += QLatin1String(", ");
@@ -1469,9 +1558,9 @@ void EditEmailAlarmDlg::slotAddAttachment()
 */
 void EditEmailAlarmDlg::slotRemoveAttachment()
 {
-    int item = mEmailAttachList->currentIndex();
+    const int item = mEmailAttachList->currentIndex();
     mEmailAttachList->removeItem(item);
-    int count = mEmailAttachList->count();
+    const int count = mEmailAttachList->count();
     if (item >= count)
         mEmailAttachList->setCurrentIndex(count - 1);
     if (!count)
@@ -1518,7 +1607,7 @@ EditAudioAlarmDlg::EditAudioAlarmDlg(bool Template, const KAEvent* event, bool n
 {
     qCDebug(KALARM_LOG) << "EditAudioAlarmDlg: Event.id()";
     init(event);
-    mTryButton->setEnabled(!MessageWin::isAudioPlaying());
+    mTryButton->setEnabled(!MessageDisplay::isAudioPlaying());
     connect(theApp(), &KAlarmApp::audioPlaying, this, &EditAudioAlarmDlg::slotAudioPlaying);
 }
 
@@ -1536,6 +1625,8 @@ QString EditAudioAlarmDlg::type_caption() const
 */
 void EditAudioAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 {
+    mTryButton->setWhatsThis(i18nc("@info:whatsthis", "Play the audio file now"));
+    mTryButton->setToolTip(i18nc("@info:tooltip", "Play the audio file now"));
     // File name edit box
     mSoundConfig = new SoundWidget(false, true, parent);
     if (isTemplate())
@@ -1545,7 +1636,7 @@ void EditAudioAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 
     // Top-adjust the controls
     mPadding = new QWidget(parent);
-    QHBoxLayout* hlayout = new QHBoxLayout(mPadding);
+    auto* hlayout = new QHBoxLayout(mPadding);
     hlayout->setContentsMargins(0, 0, 0, 0);
     hlayout->setSpacing(0);
     frameLayout->addWidget(mPadding);
@@ -1636,15 +1727,15 @@ bool EditAudioAlarmDlg::type_stateChanged() const
 * Extract the data in the dialog specific to the alarm type and set up a
 * KAEvent from it.
 */
-void EditAudioAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, const QString& text, int lateCancel, bool trial)
+void EditAudioAlarmDlg::type_setEvent(KAEvent& event, const KADateTime& dt, const QString& name, const QString& text, int lateCancel, bool trial)
 {
     Q_UNUSED(text);
     Q_UNUSED(trial);
-    event = KAEvent(dt, QString(), QColor(), QColor(), QFont(), KAEvent::AUDIO, lateCancel, getAlarmFlags());
+    event = KAEvent(dt, name, QString(), QColor(), QColor(), QFont(), KAEvent::AUDIO, lateCancel, getAlarmFlags());
     float volume, fadeVolume;
     int   fadeSecs;
     mSoundConfig->getVolume(volume, fadeVolume, fadeSecs);
-    int   repeatPause = mSoundConfig->repeatPause();
+    const int repeatPause = mSoundConfig->repeatPause();
     QUrl url;
     mSoundConfig->file(url, false);
     event.setAudioFile(url.toString(), volume, fadeVolume, fadeSecs, repeatPause, isTemplate());
@@ -1682,12 +1773,12 @@ bool EditAudioAlarmDlg::checkText(QString& result, bool showErrorMessage) const
 */
 void EditAudioAlarmDlg::slotTry()
 {
-    if (!MessageWin::isAudioPlaying())
+    if (!MessageDisplay::isAudioPlaying())
         EditAlarmDlg::slotTry();   // play the audio file
-    else if (mMessageWin)
+    else if (mMessageWindow)
     {
-        mMessageWin->stopAudio();
-        mMessageWin = nullptr;
+        MessageDisplay::stopAudio();
+        mMessageWindow = nullptr;
     }
 }
 
@@ -1696,11 +1787,11 @@ void EditAudioAlarmDlg::slotTry()
 */
 void EditAudioAlarmDlg::type_executedTry(const QString&, void* result)
 {
-    mMessageWin = (MessageWin*)result;    // note which MessageWin controls the audio playback
-    if (mMessageWin)
+    mMessageWindow = (MessageWindow*)result;    // note which MessageWindow controls the audio playback
+    if (mMessageWindow)
     {
         slotAudioPlaying(true);
-        connect(mMessageWin, &MessageWin::destroyed, this, &EditAudioAlarmDlg::audioWinDestroyed);
+        connect(mMessageWindow, &QObject::destroyed, this, &EditAudioAlarmDlg::audioWinDestroyed);
     }
 }
 
@@ -1716,9 +1807,9 @@ void EditAudioAlarmDlg::slotAudioPlaying(bool playing)
         mTryButton->setEnabled(true);
         mTryButton->setCheckable(false);
         mTryButton->setChecked(false);
-        mMessageWin = nullptr;
+        mMessageWindow = nullptr;
     }
-    else if (mMessageWin)
+    else if (mMessageWindow)
     {
         // The test sound file is playing, so enable the Try button and depress it
         mTryButton->setEnabled(true);
@@ -1742,7 +1833,7 @@ void EditAudioAlarmDlg::slotAudioPlaying(bool playing)
 CommandEdit::CommandEdit(QWidget* parent)
     : QWidget(parent)
 {
-    QVBoxLayout* vlayout = new QVBoxLayout(this);
+    auto* vlayout = new QVBoxLayout(this);
     vlayout->setContentsMargins(0, 0, 0, 0);
     mTypeScript = new CheckBox(EditCommandAlarmDlg::i18n_chk_EnterScript(), this);
     mTypeScript->setFixedSize(mTypeScript->sizeHint());
@@ -1811,7 +1902,7 @@ QString CommandEdit::text() const
 */
 QString CommandEdit::text(EditAlarmDlg* dlg, bool showErrorMessage) const
 {
-    QString result = text();
+    const QString result = text();
     if (showErrorMessage  &&  result.isEmpty())
         KAMessageBox::sorry(dlg, i18nc("@info", "Please enter a command or script to execute"));
     return result;
@@ -1853,7 +1944,7 @@ void CommandEdit::slotCmdScriptToggled(bool on)
 */
 QSize CommandEdit::minimumSizeHint() const
 {
-    QSize t(mTypeScript->minimumSizeHint());
+    const QSize t(mTypeScript->minimumSizeHint());
     QSize s(mCommandEdit->minimumSizeHint().expandedTo(mScriptEdit->minimumSizeHint()));
     s.setHeight(s.height() + style()->pixelMetric(QStyle::PM_LayoutVerticalSpacing) + t.height());
     if (s.width() < t.width())
@@ -1870,7 +1961,7 @@ QSize CommandEdit::minimumSizeHint() const
 TextEdit::TextEdit(QWidget* parent)
     : KTextEdit(parent)
 {
-    QSize tsize = sizeHint();
+    QSize tsize = TextEdit::minimumSizeHint();   // avoid calling virtual method from constructor
     tsize.setHeight(fontMetrics().lineSpacing()*13/4 + 2*frameWidth());
     setMinimumSize(tsize);
 }
