@@ -1,21 +1,9 @@
 /*
  *  sounddlg.cpp  -  sound file selection and configuration dialog and widget
  *  Program:  kalarm
- *  Copyright © 2005-2020 David Jarvie <djarvie@kde.org>
+ *  SPDX-FileCopyrightText: 2005-2020 David Jarvie <djarvie@kde.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "sounddlg.h"
@@ -60,7 +48,7 @@ SoundDlg::SoundDlg(const QString& file, float volume, float fadeVolume, int fade
                    const QString& caption, QWidget* parent)
     : QDialog(parent)
 {
-    QVBoxLayout* layout = new QVBoxLayout(this);
+    auto* layout = new QVBoxLayout(this);
     mSoundWidget = new SoundWidget(true, true, this);
     layout->addWidget(mSoundWidget);
 
@@ -150,7 +138,7 @@ QString SoundWidget::mDefaultDir;
 SoundWidget::SoundWidget(bool showPlay, bool showRepeat, QWidget* parent)
     : QWidget(parent)
 {
-    QVBoxLayout* layout = new QVBoxLayout(this);
+    auto* layout = new QVBoxLayout(this);
 
     QLabel* label = nullptr;
     if (!showPlay)
@@ -161,7 +149,7 @@ SoundWidget::SoundWidget(bool showPlay, bool showRepeat, QWidget* parent)
 
     QWidget* box = new QWidget(this);
     layout->addWidget(box);
-    QHBoxLayout* boxHLayout = new QHBoxLayout(box);
+    auto* boxHLayout = new QHBoxLayout(box);
     boxHLayout->setContentsMargins(0, 0, 0, 0);
     boxHLayout->setSpacing(0);
 
@@ -210,7 +198,7 @@ SoundWidget::SoundWidget(bool showPlay, bool showRepeat, QWidget* parent)
         mRepeatGroupBox->setWhatsThis(i18nc("@info:whatsthis", "If checked, the sound file will be played repeatedly for as long as the message is displayed."));
         connect(mRepeatGroupBox, &GroupBox::toggled, this, &SoundWidget::changed);
         layout->addWidget(mRepeatGroupBox);
-        QVBoxLayout* glayout = new QVBoxLayout(mRepeatGroupBox);
+        auto* glayout = new QVBoxLayout(mRepeatGroupBox);
 
         // Pause between repetitions
         QWidget* box = new QWidget(mRepeatGroupBox);
@@ -225,7 +213,7 @@ SoundWidget::SoundWidget(bool showPlay, bool showRepeat, QWidget* parent)
         mRepeatPause->setSingleShiftStep(10);
         mRepeatPause->setFixedSize(mRepeatPause->sizeHint());
         label->setBuddy(mRepeatPause);
-        connect(mRepeatPause, static_cast<void (SpinBox::*)(int)>(&SpinBox::valueChanged), this, &SoundWidget::changed);
+        connect(mRepeatPause, &SpinBox::valueChanged, this, &SoundWidget::changed);
         label = new QLabel(i18nc("@label", "seconds"), box);
         boxHLayout->addWidget(label);
         label->setFixedSize(label->sizeHint());
@@ -235,7 +223,7 @@ SoundWidget::SoundWidget(bool showPlay, bool showRepeat, QWidget* parent)
     // Volume
     QGroupBox* group = new QGroupBox(i18nc("@title:group Sound volume", "Volume"), this);
     layout->addWidget(group);
-    QGridLayout* grid = new QGridLayout(group);
+    auto* grid = new QGridLayout(group);
     grid->setColumnStretch(2, 1);
     const int indentWidth = CheckBox::textIndent(this) - grid->horizontalSpacing();
     grid->setColumnMinimumWidth(0, indentWidth);
@@ -282,7 +270,7 @@ SoundWidget::SoundWidget(bool showPlay, bool showRepeat, QWidget* parent)
     mFadeTime->setSingleShiftStep(10);
     mFadeTime->setFixedSize(mFadeTime->sizeHint());
     label->setBuddy(mFadeTime);
-    connect(mFadeTime, static_cast<void (SpinBox::*)(int)>(&SpinBox::valueChanged), this, &SoundWidget::changed);
+    connect(mFadeTime, &SpinBox::valueChanged, this, &SoundWidget::changed);
     label = new QLabel(i18nc("@label", "seconds"), mFadeBox);
     boxHLayout->addWidget(label);
     label->setFixedSize(label->sizeHint());
@@ -449,7 +437,7 @@ void SoundWidget::playSound()
     mPlayer->setParent(this);
 #else
     mPlayer = new Phonon::MediaObject(this);
-    Phonon::AudioOutput* output = new Phonon::AudioOutput(Phonon::MusicCategory, mPlayer);
+    auto* output = new Phonon::AudioOutput(Phonon::MusicCategory, mPlayer);
     mPlayer->setCurrentSource(mUrl);
     Phonon::createPath(mPlayer, output);
 #endif
