@@ -1,21 +1,9 @@
 /*
  *  timeselector.cpp  -  widget to optionally set a time period
  *  Program:  kalarm
- *  Copyright © 2004-2016 David Jarvie <djarvie@kde.org>
+ *  SPDX-FileCopyrightText: 2004-2016 David Jarvie <djarvie@kde.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "timeselector.h"
@@ -33,7 +21,7 @@ TimeSelector::TimeSelector(const QString& selectText, const QString& selectWhats
                            const QString& valueWhatsThis, bool allowHourMinute, QWidget* parent)
     : QFrame(parent)
 {
-    QHBoxLayout* layout = new QHBoxLayout(this);
+    auto* layout = new QHBoxLayout(this);
     layout->setContentsMargins(0, 0, 0, 0);
     mSelect = new CheckBox(selectText, this);
     mSelect->setFixedSize(mSelect->sizeHint());
@@ -43,7 +31,7 @@ TimeSelector::TimeSelector(const QString& selectText, const QString& selectWhats
 
     QWidget* box = new QWidget(this);    // to group widgets for QWhatsThis text
     layout->addWidget(box);
-    QHBoxLayout* boxLayout = new QHBoxLayout(box);
+    auto* boxLayout = new QHBoxLayout(box);
     boxLayout->setContentsMargins(0, 0, 0, 0);
     mPeriod = new TimePeriod(allowHourMinute, box);
     boxLayout->addWidget(mPeriod);
